//****************************************************************************
//****************************************************************************
//***     RCA CDP1833 ROM
//***          Dumper
//***    December 31, 2016
//***         Ed Keefe
//****************************************************************************
//****************************************************************************
//
//   The RCA CDP1833 is a 1K x 8 bit ROM
//   It has multiplexed address lines for easy interfacing with the RCA 1802 
//   microprocessor.
//   
//   It has mask-programmable chip select lines. For this reason the CS1 and CS2 //   lines are connected to jumpers. (See schematic)
//   
//   The MUX line is used to latch the high address into the ROMs internal
//   latch. This is normally connected to the TPA line of the 1802. 
//   
//   This program simulates the addressing operations of the 1802 for the 
//   purpose of reading the contents of a CDP1833 ROM.
//   
//   It does this by cycling through the entire 64K memory space and sending 
//   the contents of each address out the Arduino UNO's serial port to the PC.
//   
//   The user wires the ROM to the Arduino UNO as shown in the schematic and 
//   sets the two jumpers to a known state. The schematic shows them both set 
//   for 'LOW'. 
//   
//   The user then starts a terminal program set to 9600 N81 on the same 
//   comport as the Arduino UNO.
//
//   The user then powers up the Arduino with the ROM attached and waits until 
//   the terminal displays 'DONE.'
//   
//   The user would then scan this output for a 1K block of random data. The 
//   other 63K blocks output will contain just 'FF's.
//
//   If all data is '00's, move one of the jumpers and repeat the process. 
//   There are four possible settings for the two jumpers.
//   
//   If that doesn't work, the MUX signal may need to be inverted. The MUX pin
//   on the CDP1833 is also mask-programmable. The 'TPA' variable in this
//   program is used to control the 'MUX' pin. Here is how to invert the MUX 
//   signal. 
//
//   In the 'void loop()' routine there are three lines that use 'TPA'. These
//   need their states inverted. 
//
//   If a line shows 'digitalWrite(TPA, LOW);', 
//   change it to 'digitalWrite(TPA, HIGH);'
//
//   If a line shows 'digitalWrite(TPA, HIGH);',
//   change it to 'digitalWrite(TPA, LOW);;'
//
//   Repeat the steps above again.
//
//   Good luck.

//The constants below are the 'Pin' Numbers on an Arduino UNO

int Data0           = 2;
int Data1           = 3;
int Data2           = 4;
int Data3           = 5;
int Data4           = 6;
int Data5           = 7;
int Data6           = 8;
int Data7           = 9;
int Address0        = 10;
int Address1        = 11;
int Address2        = 12;
int Address3        = 13;
int Address4        = 14;
int Address5        = 15;
int Address6        = 16;
int Address7        = 17;
int TPA             = 18;
int MRD             = 19;

//Variables
int ReadInData      = 0;
int ByteCount       = 0;
int DataPrintWidth  = 8;

void setup() 
  {
  Serial.begin(9600);
  pinMode(Data0,    INPUT_PULLUP);
  pinMode(Data1,    INPUT_PULLUP);
  pinMode(Data2,    INPUT_PULLUP);
  pinMode(Data3,    INPUT_PULLUP);
  pinMode(Data4,    INPUT_PULLUP);
  pinMode(Data5,    INPUT_PULLUP);
  pinMode(Data6,    INPUT_PULLUP);
  pinMode(Data7,    INPUT_PULLUP);
  pinMode(Address0, OUTPUT);
  pinMode(Address1, OUTPUT);
  pinMode(Address2, OUTPUT);
  pinMode(Address3, OUTPUT);
  pinMode(Address4, OUTPUT);
  pinMode(Address5, OUTPUT);
  pinMode(Address6, OUTPUT);
  pinMode(Address7, OUTPUT);
  pinMode(TPA,      OUTPUT);
  pinMode(MRD,      OUTPUT);
  }

void loop() 
  {
  Serial.println("Start...");
  for (int AddressHigh = 0x00; AddressHigh < 0x100; AddressHigh++)
    {
    Serial.print("----> "); 
    Serial.print(AddressHigh, HEX);
    Serial.println("00 ");
    for (int AddressLow = 0x00; AddressLow < 0x100; AddressLow++)
      {
      digitalWrite(TPA, LOW);
      digitalWrite(MRD, HIGH);
      SetAddress(AddressHigh);
      digitalWrite(TPA, HIGH);
      delay(1);
      digitalWrite(TPA, LOW);
      SetAddress(AddressLow);
      digitalWrite(MRD, LOW);
      delay(1);
      ReadData();
      digitalWrite(MRD, HIGH);
      SendData(ReadInData);
      }
    }
  Serial.print("Done.");
  while(1){}  
  }

void SetAddress(int address)
  {
  digitalWrite(Address0, LOW);
  digitalWrite(Address1, LOW);
  digitalWrite(Address2, LOW);
  digitalWrite(Address3, LOW);
  digitalWrite(Address4, LOW);
  digitalWrite(Address5, LOW);
  digitalWrite(Address6, LOW);
  digitalWrite(Address7, LOW);

  if (address & 0b00000001) digitalWrite(Address0, HIGH);
  if (address & 0b00000010) digitalWrite(Address1, HIGH);
  if (address & 0b00000100) digitalWrite(Address2, HIGH);
  if (address & 0b00001000) digitalWrite(Address3, HIGH);
  if (address & 0b00010000) digitalWrite(Address4, HIGH);
  if (address & 0b00100000) digitalWrite(Address5, HIGH);
  if (address & 0b01000000) digitalWrite(Address6, HIGH);
  if (address & 0b10000000) digitalWrite(Address7, HIGH);
  }
  
void ReadData()
  {
  ReadInData = 0x00;
  if (digitalRead(Data0) == HIGH) ReadInData += 1;
  if (digitalRead(Data1) == HIGH) ReadInData += 2;
  if (digitalRead(Data2) == HIGH) ReadInData += 4;
  if (digitalRead(Data3) == HIGH) ReadInData += 8;
  if (digitalRead(Data4) == HIGH) ReadInData += 16;
  if (digitalRead(Data5) == HIGH) ReadInData += 32;
  if (digitalRead(Data6) == HIGH) ReadInData += 64;
  if (digitalRead(Data7) == HIGH) ReadInData += 128;
  }

void SendData(int x)
  {
  if (x < 16) Serial.print("0");
  Serial.print(x,HEX);
  Serial.print(" ");
  ByteCount++;
  if (ByteCount == DataPrintWidth)
    {
    Serial.println(" ");
    ByteCount = 0;
    }
  }
